import cv2
import board
from digitalio import DigitalInOut, Direction
from time import sleep
import threading

# 创建VideoCapture对象，参数为摄像头索引（一般为0）
cap = cv2.VideoCapture(1)

# 检查摄像头是否成功打开
if not cap.isOpened():
    print("无法打开摄像头")
    exit()

# 初始化LED对象和设置引脚
led = DigitalInOut(board.LED)
led.direction = Direction.OUTPUT

# 初始化变量
prev_frame = None
motion_detected = False
led_state = False
lock = threading.Lock()

# LED闪烁线程函数
def led_blink():
    while True:
        with lock:
            led.value = 0 if led_state else 1
        sleep(0.5)

# 创建并启动LED闪烁线程
led_thread = threading.Thread(target=led_blink)
led_thread.start()

# 循环读取摄像头的帧
while True:
    # 读取摄像头的帧
    ret, frame = cap.read()

    # 检查帧是否成功读取
    if not ret:
        print("无法读取摄像头的帧")
        break

    # 转换为灰度图像
    gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)

    # 进行动作检测
    if prev_frame is not None:
        # 计算当前帧与上一帧的差异
        frame_diff = cv2.absdiff(prev_frame, gray)

        # 对差异图像进行阈值处理
        _, thresh = cv2.threshold(frame_diff, 60, 255, cv2.THRESH_BINARY)

        # 统计非零像素的数量
        motion_detected = cv2.countNonZero(thresh) > 0

    # 更新上一帧
    prev_frame = gray

    # 根据动作检测结果设置LED状态
    with lock:
        if motion_detected:
            led_state = True
        else:
            led_state = False

    # 在窗口中显示帧
    cv2.imshow("Camera", frame)

    # 检测按键
    key = cv2.waitKey(1)
    if key == ord("q"):
        break

# 释放摄像头资源
cap.release()

# 关闭窗口
cv2.destroyAllWindows()

# 等待LED闪烁线程结束
led_thread.join()